package RPM::Source::Dependency::Analyzer::Collector::Parser;

#package SourceAnalyzer::Language;

use strict;
use warnings;
use Cwd;
use File::Basename;
our $VERSION=0.001;

sub new {
    my $class = shift;
    my $self = {
	-verbose => 0,
	@_
    };
    bless $self, $class;
}

sub parse_file {
    my ($self,$file)=@_;
    if (not -e $file and -l $file) {
	# known; for example, libyui-gtk-2.44.10 has
	# CMakeLists.txt -> /home/angelo/local/share/libyui/buildtools/CMakeLists.common
	print STDERR "INFO: Source Analyzer: ignored broken link $file\n" if $self->{verbose};
	return;
    }
    my $content = &__load_file($file);
    return $self->parse_string($content,$file);
}

sub __load_file {
    my ($file)=@_;
    open my $fh, "<", $file or do {
	print STDERR "Internal error: __load_file: when opening $file:\nin curdir=",Cwd::getcwd()," got error: $!\n";
	my $path=$file;
	while (not -e $path) {
	    print STDERR "path that not found: [$path]\n";
	    $path=dirname($path);
	}
	print STDERR "top path that found: $path\nls -l:\n";
	system('ls', '-l' ,$path);
	Carp::confess();
    };
    local $/; # enable localized slurp mode
    my $content = <$fh>;
    close $fh;
    return $content;
}


# call when all files are processed
sub finish {}


## used in AutoMake
sub _raw_collect_handler {
    my $parser=shift;
    my $class=shift;
    my $type=shift;
    my $storage=$parser->{-storage};
    my $verbose=$parser->{-verbose};
    #print STDERR "_raw_collect_handler: P:$parser S:$storage T:$type ".join(' ',$@)."\n";
    if ($verbose && $verbose>3) {
	print "_raw_collect_handler: $class $type ".join(' ',$@)."\n";
    }
    $storage->add_raw_storage_deps($class,$type, @_);
}

1;
