package RPM::Source::Dependency::Analyzer::Collector::Parser::AutoMake;
#package SourceAnalyzer::Language::AutoMake;

use strict;
use warnings;

use RPM::Source::Dependency::Analyzer::Collector::Parser;

our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser/;
our $verbose=0;
our $VERSION=0.001;

sub parse_string {
    my ($self,$data)=@_;
    $data=~s/\\\n//gs;
    foreach my $line (split("\n",$data)) {
	&_parse_line($self,$line);
    }
    my $hook=$self->{-postfilehook};
    &$hook($self) if $hook;
}

sub _parse_line {
    my ($self,$line)=@_;
    # Makefile is line-censitive
    #$line=~s/^\s*//;
    $line=~s/\s*$//;
    #print STDERR "LINE=$line\n";
    if ($line=~s/^AM_VALAFLAGS\s*=\s*//) {
	#print STDERR "AM_VALAFLAGS=$line\n";
	my @tokens=split(/\s+/,$line);
	while (my $token = shift @tokens) {
	    #print STDERR "AM_VALAFLAGS: token=$token\n";
	    if ($token=~/^--pkg=(.*)$/) {
		#print STDERR "  pkg=$1\n";
		$self->_raw_collect_handler('vapi', 'req', $1);
	    } elsif ($token eq '--pkg') {
		$token = shift @tokens;
		#print STDERR "AM_VALAFLAGS: vapi=$token\n";
		$self->_raw_collect_handler('vapi', 'req', $token) if $token;
	    } elsif ($token=~/^--gir=(.*)$/) {
		#print STDERR "  gir=$1\n";
		$self->_raw_collect_handler('girs', 'req', $1);
	    } elsif ($token eq '--gir') {
		$token = shift @tokens;
		#print STDERR "AM_VALAFLAGS: gir=$token\n";
		$self->_raw_collect_handler('girs', 'req', $token) if $token;
	    }
	}
    } elsif ($line=~/^([\w_][\w\d_@]*)\s*\+?=\s*(.*)$/) {
	$self->add_var($1,$2);
    } elsif ($line=~/^([\w_][\w\d_@]*):\s*(.*)$/) {
	# target
    } elsif ($line=~s/^.*\s\$\((?:G_IR|INTROSPECTION)_SCANNER\)//) {
	# $(G_IR_SCANNER) $(INTROSPECTION_SCANNER)
	# cmds
	#print STDERR "girline=$line\n";
	foreach my $token (split(/\s+/,$line)) {
	    #print STDERR "  token=$token\n";
	    if ($token=~/^--include=(.*)$/) {
		#print STDERR "  include=$1\n";
		if ($1 eq 'Gtk-${GTK_API_VERSION}' or $1 eq 'Gtk-$(GTK_API_VERSION)') {
		    $self->_raw_collect_handler('girs', 'req', 'Gtk-2.0');
		    $self->_raw_collect_handler('girs', 'req', 'Gtk-3.0');
		} else {
		    $self->_raw_collect_handler('girs', 'req', $1);
		}
	    } elsif ($token=~/^--pkg=(.*)$/) {
		#print STDERR "  pkg=$1\n";
		if ($1 eq 'gtk+-${GTK_API_VERSION}' or $1 eq 'gtk+-$(GTK_API_VERSION)') {
		    $self->_raw_collect_handler('pkgconf', 'req', 'gtk+-2.0');
		    $self->_raw_collect_handler('pkgconf', 'req', 'gtk+-3.0');
		} else {
		    $self->_raw_collect_handler('pkgconf', 'req', $1);
		}
	    }
	}
    }
}

sub add_var {
    my ($self,$name,$val)=@_;
    print STDERR "VAR: add $name=$val\n" if $verbose>1;
    my $hook=$self->{-varhook};
    &$hook($name,$val,$self) if $hook;
    return $val;
}

1;
