package RPM::Source::Dependency::Analyzer::Collector::Parser::CMake::Marpa;

use strict;
use warnings;

use Marpa::R2;

#action => call_dump
#action => ::first
#action => process_call
our $dsl=<<'END_OF_SOURCE';
lexeme default = latm => 1

#:start    ::= language
#language  ::= Expressions

:start    ::= Expressions
Expressions ::= Expression*
Expression ::=
    Discardables Expression			action => return_second
    | Expression Discardables			action => return_first
    | QQuotedString          			action => ::first
    | CMD_Identifier '(' Expressions ')'  	
    | CMD_Identifier  '(' ')'  			
    | CMD_Identifier Discardables '(' Expressions ')'	action => skip_second
    | CMD_Identifier Discardables '(' ')'       	action => skip_second
    | Something_Unquoted 			action => ::first
# or something like a/"b" will be split :(
    | Something_Unquoted Something_Unquoted	action => _concat
    | Something_Unquoted QQuotedString		action => _concat
    | QQuotedString Something_Unquoted		action => _concat
# should normally not happen, but let us be safe
    | '(' Expressions ')' # assoc => group 
# happen in empty/commented files
    | Discardables        action => ::first

Discardables ::= Discardable+ #action => _concat
Discardable ::= <hash comment> action => ::first | whitespace action => ::first

QQuotedString ::= 
      '"' '"'                   action => process_empty_quoted_string
    | '"' qquotedstringbody '"' action => process_quoted_string

qquotedstringbody ::= qquotedstringbodychar+ action => _concat
qquotedstringbodychar ~ [\s()#] | valuechar | escaped_char
# valuechar can have non-ascii characters too!!
valuechar ~ [^\s()#"\\]  # "
escaped_char ~ '\' [\S\s] # '
	
CMD_Identifier      ~ [_a-zA-Z] id_rest
id_rest         ~ [_0-9a-zA-Z]*

Something_Unquoted ~ unquotedstringchar+
unquotedstringchar ~ valuechar | escaped_char


## added whitespace to Expression instead of discard
#:discard ~ whitespace
whitespace ~ [\s]+
# allow comments
#:discard ~ <hash comment>
<hash comment> ~ <terminated hash comment> | <unterminated
   final hash comment>
<terminated hash comment> ~ '#' <hash comment body> <vertical space char>
<unterminated final hash comment> ~ '#' <hash comment body>
<hash comment body> ~ <hash comment char>*
<vertical space char> ~ [\x{A}\x{B}\x{C}\x{D}\x{2028}\x{2029}]
<hash comment char> ~ [^\x{A}\x{B}\x{C}\x{D}\x{2028}\x{2029}]
END_OF_SOURCE

our $grammar = Marpa::R2::Scanless::G->new({
    default_action => '::array',
    source         => \$RPM::Source::Dependency::Analyzer::Collector::Parser::CMake::Marpa::dsl,
});

sub process_quoted_string {
# 0={} 1=" 2=arg 3="
#print "QS [args]:\n";
#print Dumper([@_]);
    return $_[2];
}

sub process_empty_quoted_string {
    return '';
}

sub _concat {
    shift;
    return join('',@_);
}

sub skip_second {
    shift;
    my @ret;
    push @ret, shift;
    my $second=shift;
    #warn "CMAKE: Marpa grammar: internal error: expected [space], got $space" if $space!~/^\s+$/;
    push @ret,@_;
    return [@ret];
}

sub return_first {
    return $_[1];
}

sub return_second {
    return $_[2];
}


1;

__END__

