package RPM::Source::Dependency::Analyzer::Collector::Plugin::AutoMake;

use strict;
use warnings;
use Text::ParseWords qw(shellwords);

use RPM::Source::Dependency::Analyzer::Collector::Parser::AutoMake;
our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser::AutoMake/;

our $verbose=0;
our $opt_ignore_own_libraries=1;

sub new {
    my $class=shift;
    $class->SUPER::new(
	-varhook => \&_automake_var_handler,
	-verbose => $verbose,
	@_
	);
}

sub _automake_var_handler {
    my ($name,$args,$am_parser)=@_;
    my $storage=$am_parser->{-storage};
    if ($verbose) {
	print STDERR "_automake_var_handler: $name=$args\n";
    }
    if ($name=~/_(?:LIBS|LDFLAGS|LDADD|LIBADD)$/) {
	foreach my $word (shellwords($args)) {
	    $storage->add_libs_req($word) if $word=~s/^-l(?=\w)//;
	}
    } elsif ($name=~/_gir_INCLUDES$/) {
	foreach my $word (shellwords($args)) {
	    if ($word eq 'Gtk-${GTK_API_VERSION}' or $word eq 'Gtk-$(GTK_API_VERSION)') {
		$storage->add_girs_req('Gtk-2.0');
		$storage->add_girs_req('Gtk-3.0');
	    } else {
		$storage->add_girs_req($word);
	    }
	}
    } elsif ($name=~/_gir_PACKAGES$/) {
	# vala dependencies; put in /usr/share/vala/vapi/*.deps
	foreach my $word (shellwords($args)) {
	    $storage->add_vapi_req($word);
	}
    } elsif ($opt_ignore_own_libraries and $name eq 'lib_LTLIBRARIES') {
	foreach my $word (shellwords($args)) {
	    $storage->add_libs_ignore($1) if $word=~/^lib(.*)\.la$/;
	}
    }
}


1;
