package RPM::Source::Dependency::Analyzer::Collector::Plugin::Autoconf;

use strict;
use warnings;
use Carp;
use Text::ParseWords qw(shellwords);

use RPM::Source::Dependency::Analyzer::Collector::Parser::Autoconf;
our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser::Autoconf/;

our $verbose=1;

sub new {
    my $class=shift;
    $class->SUPER::new(
#	-verbose => $verbose,
	-macrohook => \&_autoconf_macro_handler,
	-shellhook => \&_autoconf_shell_handler,
	@_
	);
}

my %LANG_FLAGS=qw/
AC_PROG_CC	c
AM_PROG_CC_STDC	c
AM_PROG_LIBTOOL	c
AC_PROG_CXX	cxx
AC_PROG_CXXCPP	cxx
AC_PROG_F77	fortran
AC_LANG_C	c
AC_LANG_CPLUSPLUS	cxx
AC_LANG_FORTRAN77	fortran
/;

my %LIB_FLAGS=qw/
AM_PATH_AALIB	aa
AM_PATH_ALLEGRO	alleg
AM_PATH_ALSA	asound
AM_PATH_ASPELL	aspell
AM_PATH_AUDIOFILE	audiofile
AC_PATH_BLUETOOTH	bluetooth
AM_PATH_CHECK	check
AC_PATH_CLN	cln
AM_PATH_CPPUNIT	cppunit
AM_PATH_DVDNAV	dvdnav
AM_PATH_ESD	esd
AM_PATH_GESD	esd
AM_PATH_FLITE	flite
AM_PATH_FLTK	fltk
AM_PATH_CG	cg
AM_PATH_GINAC	ginac
AM_PATH_GMP	gmp
AM_PATH_GPG_ERROR	gpg-error
AM_PATH_GSL	gsl
AM_PATH_GTK	gtk
AM_PATH_GTK_2_0	gtk-x11-2.0
AM_PATH_GTK_3_0 gtk-3
AM_PATH_GLIB	glib
AM_PATH_GLIB_2_0	glib-2.0
AM_PATH_GLUT	glut
AM_PATH_GPGME	gpgme
AC_PATH_LAME	libmp3lame
AM_PATH_LAPACK	lapack
AM_PATH_LIB3DS	3ds
AM_PATH_LIBCDAUDIO	cdaudio
AM_PATH_LIBEXPAT	expat
AM_PATH_LIBFAME	fame
AM_PATH_LIBFLAC	flac
AM_PATH_LIBGCRYPT_CCRTP	gcrypt
AM_PATH_LIBGCRYPT	gcrypt
AM_PATH_LIBGLADE	glade-2.0
AM_PATH_LIBGNUTLS	gnutls
AM_PATH_LIBMIKMOD	mikmod
AM_PATH_LIBOTR		otr
AM_PATH_LIBPNG		png
AM_PATH_LIBPRELUDE	prelude
AM_PATH_LIBRAPI2	rapi
AM_PATH_LUA	lua
AC_PATH_MAGIC	magic
AM_PATH_MPFR	mpfr
AM_PATH_ODBC	odbc
AM_PATH_OPENAL	openal
AM_PATH_OPENGL	GL
AM_PATH_OPENOBEX	openobex
AM_PATH_PHYSFS	physfs
AM_PATH_PILOT_LINK	pisync
AM_PATH_PNG	png
AM_PATH_RADIUS_ENGINE	radius-engine
AM_PATH_READLINE	readline
AM_PATH_SC	mpqc
AM_PATH_SDL	SDL
AM_PATH_SDL_GFX	SDL_gfx
AM_PATH_SDL_IMAGE	SDL_image
AM_PATH_SDL_MIXER	SDL_mixer
AM_PATH_SDL_NET	SDL_net
AM_PATH_SDL_PANGO	SDL_pango
AM_PATH_SDL_SOUND	SDL_sound
AM_PATH_SDL_TTF	SDL_ttf
AM_PATH_SDL2	SDL2
AM_PATH_SDL2_GFX	SDL2_gfx
AM_PATH_SDL2_IMAGE	SDL2_image
AM_PATH_SDL2_MIXER	SDL2_mixer
AM_PATH_SDL2_NET	SDL2_net
AM_PATH_SDL2_PANGO	SDL2_pango
AM_PATH_SDL2_SOUND	SDL2_sound
AM_PATH_SDL2_TTF	SDL2_ttf
AM_PATH_STDAIR	stdair
AM_PATH_TALKFILTERS	talkfilters
AC_PATH_USB	usb-1.0
AM_PATH_WNN	wnn;crypt
AC_PATH_X_APP_DEFAULTS	xmkmf
AM_PATH_XFT	Xft
AM_PATH_XINE	xine
AM_PATH_XML2	xml2
AM_PATH_XMMS	xmms
AST_IMLIB2_SUPPORT	Imlib2;freetype;Xext
DAP_CHECK_ZLIB	z
DECAF_CONFIGURE_APR	expat
/;
#AC_PATH_XTRA	ICE;SM;X11 -> xorg-x11-devel?

my %HEADER_FLAGS=qw!
AC_PATH_IRDA	lib/irda.h
AC_PATH_IRDA_LINUX	lib/irda.h
AM_PATH_SYSLOG	syslog.h
AC_PATH_X	X11/Xlib.h
!;

my %PKG_FLAGS=qw!
AM_PATH_GNAT	gcc-gnat
AM_PATH_HASHDIR ispell
AC_PATH_KDE	kdelibs-devel
AM_PATH_PYGTK	python-module-pygtk-devel
AM_PATH_PYTHON	python3-devel
AM_PATH_PYTHON_VERSION	python3-devel
AC_PATH_QT_1_3	libqt3-devel
AM_PATH_RUBY	libruby-devel
AC_PATH_XTRA	imake;libXt-devel;xorg-cf-files
GLIB_GSETTINGS	libgio-devel
!;

my %PATH_FLAGS=qw!
GTK_DOC_CHECK	gtkdocize
AC_PATH_BSD_MAKE	make
AC_PATH_GNU_MAKE	gmake
AM_GLIB_GNU_GETTEXT	glib-gettextize
AM_PATH_CCACHE	ccache
AM_PATH_PERL	perl
AM_PATH_PKGCONFIG	pkg-config
AM_PATH_SQLITE	sqlite3
AM_PATH_SQLITE3	sqlite3
AM_PATH_WXCONFIG	wx-config
!;

#AM_PATH_GARTS=AM_PATH_ARTS
my %IGNORE_FLAGS=map {$_=>1} qw!
AM_PATH_LISPDIR	
AC_PATH_WIN32	
AM_PATH_DIRECTX	
AM_PATH_LIBOLE2	
AM_PATH_ARTS	
AM_PATH_GARTS	
AC_PATH_TOOL	
AC_PATH_GENERIC
!;


my %LIB_BAN=map {$_=>1} # glibc stubs
qw!socket bsd nsl inet intl!;

#AC_CHECK_LIB(nls, gethostbyname_r)
#AC_CHECK_LIB(socket, socket)
my %BAN_CHECK_LIBS;
my @_BAN_CHECK_LIBS=qw/
bsd gettimeofday
bsd optreset
net gethostbyname
nls gethostbyname_r
socket socket
xnet connect
inet connect
/;
while (@_BAN_CHECK_LIBS) {
    my $key=shift @_BAN_CHECK_LIBS;
    my $val=shift @_BAN_CHECK_LIBS;
    $BAN_CHECK_LIBS{$key}{$val}=1;
}

# glibc
my %BAN_SEARCH_LIBS=map {$_=>1} qw/
gethostbyname gethostbyaddr sethostent gethostent endhostent h_errno herror hstrerror 
gethostbyaddr_r gethostbyname2 gethost‐byname2_r gethostbyname_r gethostent_r
gethostname sethostname
inet_aton inet_addr inet_network inet_ntoa inet_makeaddr inet_lnaof inet_netof

connect accept bind getsockname listen socket getsockopt setsockopt
getaddrinfo freeaddrinfo gai_strerror

gettimeofday settimeofday
/;

#GNOME_COMMON_INIT
## Docs are disabled for now
##GNOME_DOC_INIT
#GNOME_MAINTAINER_MODE_DEFINES
#GNOME_COMPILE_WARNINGS
#GNOME_DEBUG_CHECK

# TODO: implicit AC_TRY_LINK
#LIBS="-lutil"
#AC_TRY_LINK(, , [needutil=yes], [needutil=no])

# dep on GConf
#AM_GCONF_SOURCE_2

sub _autoconf_shell_handler {
    my ($name,$args,$ac_parser)=@_;
    print STDERR "_autoconf_macro_handler: $name=[".join(',',@$args)."]\n" if $verbose>3;
    my $storage=$ac_parser->{-storage};
    my $vars=$ac_parser->{-vardata};
    if ($name eq 'pkg-config') {
	# $PKG_CONFIG --atleast-version $XRANDR_REQUIRED xrandr
	if ($args->[0] eq '--atleast-version') {
	    shift @$args;
	    shift @$args;
	} elsif ($args->[0] eq '--exists') {
	    shift @$args;
	}
	if (@$args>1) {
	    warn "SourceAnalyzer: unrecognized pkg-config args: ".join(' ',@$args);
	}
	$storage->add_pkgconf_req( &__subst_vars($args->[0],$vars) );
    } else {
	warn "SourceAnalyzer: shell handler: unrecognized command: $name ".join(' ',@$args);
    }
}

sub _autoconf_macro_handler {
    my ($name,$args,$ac_parser)=@_;
    print STDERR "_autoconf_macro_handler: $name=[".join(',',@$args)."]\n" if $verbose>3;
    my $storage=$ac_parser->{-storage};
    my $vars=$ac_parser->{-vardata};
    my $lang_flag=$LANG_FLAGS{$name};
    my $lib_flag=$LIB_FLAGS{$name};
    my $pkg_flag=$PKG_FLAGS{$name};
    my $path_flag=$PATH_FLAGS{$name};
    my $header_flag=$HEADER_FLAGS{$name};
    $storage->set_lang_flag($lang_flag) if $lang_flag;
    $storage->add_libs_req( split(';',$lib_flag) ) if $lib_flag;
    $storage->add_pkgs_req( split(';',$pkg_flag) ) if $pkg_flag;
    $storage->add_path_req( split(';',$path_flag) ) if $path_flag;
    $storage->add_hdrs_req( &__add_h(split(';',$header_flag)) ) if $header_flag;
    warn "SourceAnalyzer: unknown macros $name. Please, report.\n" if $name=~/^A[CM]_PATH_(?!(?:PROG|GENERIC))/ and not ($lang_flag or $path_flag or $lib_flag or $header_flag or $pkg_flag or $IGNORE_FLAGS{$name});

    if ($name eq 'AC_LANG') {
	my $lang=&__strip($args->[0]);
	if ($lang eq 'C') {
	    $storage->set_lang_flag('c');
	} elsif ($lang eq 'C++') {
	    $storage->set_lang_flag('cxx');
	} elsif ($lang =~ '^Fortran') {
	    $storage->set_lang_flag('fortran');
	} else {
	    warn "SourceAnalyzer bug: unknown language in AC_LANG($lang). Please, report.\n";
	}
    } elsif ($name eq 'AC_CHECK_FILE') {
	my $file=&__strip($args->[0]);
	if ($file =~ /(docbook|titlepage).xsl/) {
	    $storage->add_pkgs_req( 'docbook-dtds', 'docbook-style-xsl' );
	} elsif ($file =~ m!^/proc!) {
	    $storage->add_pkgs_req( '/proc' );
	} else {
	    warn "SourceAnalyzer: AC_CHECK_FILE '$args->[0]'\n";
	}
    } elsif ($name eq 'AC_CHECK_LIB') {
	#my $verbose=9;
	my $lib=&__strip($args->[0]);
	my $func;
	$func=&__strip($args->[1]) if defined($args->[1]);
	print STDERR "AC_CHECK_LIB: 1st arg $lib\n" if $verbose>3;
	print STDERR "AC_CHECK_LIB: 2nd arg $func\n" if defined($func) and $verbose>3;
	my $banhash=$BAN_CHECK_LIBS{$lib};
	print STDERR "AC_CHECK_LIB: $lib has ban\n" if $banhash and $verbose>3;
	if (defined ($func) and defined($banhash)) {
	    if ($banhash->{$func} or $BAN_SEARCH_LIBS{$func} or $LIB_BAN{$lib}) {
		# ban
	    } else {
		print STDERR "INFO: AC_CHECK_LIB($lib, $func): platform specific: check manually.\n" if $verbose;
		$storage->add_libs_req( &__subst_vars($lib,$vars));
	    }
	} else {
	    $storage->add_libs_req( &__subst_vars($lib,$vars));
	}
    } elsif ($name eq 'AC_SEARCH_LIBS') {
	my $func=&__strip($args->[0]);
	if (not $BAN_SEARCH_LIBS{$func}) {
	    $storage->add_libs_req( [&__split(&__subst_vars($args->[1],$vars))]);
	}
    } elsif ($name eq 'AC_CHECK_HEADER') {
	$storage->add_hdrs_req( &__add_h(&__subst_vars($args->[0], $vars)));
    } elsif ($name eq 'AC_CHECK_HEADERS') {
	$storage->add_hdrs_req( &__add_h(&__split(&__subst_vars($args->[0],$vars))));
    } elsif ($name eq 'AC_PATH_GENERIC') {
	$storage->add_path_req( map {$_.'-config'} &__subst_vars($args->[1],$vars));
    } elsif ($name eq 'AC_PATH_PROG') {
	my $path=$args->[1];
	if (defined $path) {
	    $storage->add_path_req( &__subst_vars(&__unbacktick($path), $vars) );
	} else {
	    warn "SourceAnalyzer: strange AC_PATH_PROG ($args->[0]) - no second arg is given"
	}
    } elsif ($name eq 'AC_CHECK_PROG') {
	$storage->add_path_req( &__subst_vars(&__unbacktick($args->[1]),$vars) );
    } elsif ($name eq 'AC_CHECK_PROGS') {
	my @progs=&__split(&__subst_vars($args->[1],$vars));
	if (@progs>1) {
	    # multiple alternatives to find
	    $storage->add_path_req(\@progs);
	} else {
	    # one or less program, no alternatives
	    $storage->add_path_req(&__unbacktick(@progs));
	}
    } elsif ($name eq 'JH_CHECK_XML_CATALOG') {
	if ($args->[0] =~ /DTD DocBook/) {
	    $storage->add_pkgs_req('docbook-dtds');
	} elsif ($args->[0] =~ /docbook.xsl/) {
	    $storage->add_pkgs_req('docbook-style-xsl');
	} else {
	    warn "SourceAnalyzer: unrecognized XML catalog '$args->[0]'\n";
	}
    } elsif ($name eq 'PKG_CHECK_MODULES') {
# note 3 arg
#PKG_CHECK_MODULES([ENCHANT], [enchant], [AC_DEFINE(HAVE_LIBENCHANT, , Define if we use enchant)], [])
# TODO: multiline (see assogiate)
#PKG_CHECK_MODULES([ASSOGIATE], [                                                
#        glib-2.0 >= 2.8.0
#        glibmm-2.4 >= 2.8.0
#        gtkmm-2.4 >= 2.6.0
#        libxml++-2.6
#        gnome-vfsmm-2.6
#])                                                                              
# TODO: variable (see xqf); note: it is inside if; so it still might be better to ignore
#        pkg_modules="gtk+-2.0 >= 2.0.0 gdk-pixbuf-xlib-2.0"
#        PKG_CHECK_MODULES(PACKAGE, [$pkg_modules])
	my $pkgline=$args->[1];
	&__parse_pkg_config_line($pkgline,$vars,$storage);
    } elsif ($name eq 'PKG_CHECK_EXISTS') {
	# PKG_CHECK_EXISTS(MODULES, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
	my $pkgline=$args->[0];
	#print STDERR "PKG_CHECK_EXISTS: $args->[0]\n";# TODO debug
	&__parse_pkg_config_line($pkgline,$vars,$storage);
    }

}

sub __parse_pkg_config_line {
    my ($pkgline,$vars,$storage)=@_;
    my @raw_pkgconf;
    while ($pkgline) {
	my $item = &__read_m4_arg(\$pkgline);
	#print STDERR "__parse_pkg_config_line: [$item] \n";

	if (not $item) {
	    # skip
	} elsif ($item=~/^.+\$([\w_][\w_\d]*)/) {
	    if (not $vars->{$1}) {
		#print STDERR "__parse_pkg_config_line: push [$item]\n";
		push @raw_pkgconf, $item;
	    } else {
		#print STDERR "var: $1 - ",join(':',@{$vars->{$1}}),"\n";
		foreach (@{$vars->{$1}}) {
		    my $expanded_item=$item;
		    $expanded_item=~ s/^.+\K\$([\w_][\w_\d]*)/$_/;
		    #print STDERR "__parse_pkg_config_line: expanded push [$expanded_item]\n";
		    push @raw_pkgconf, $expanded_item;
		}
	    }
	} elsif ($item =~ /^[<>=!]/) {
	    my $discarded=&__read_m4_arg(\$pkgline);
	    #print STDERR "__parse_pkg_config_line: ignored [$item] [$discarded]\n";
	} else {
	    push @raw_pkgconf, $item;
	}
    }

    ### hack! just fix __read_m4_arg and add directly to $storage;
    while (@raw_pkgconf>0) {
	my $in=shift @raw_pkgconf;
	next if $in =~ /^\$/;
	$in =~s/^['"]//;
	$in =~s/['"]$//;
	if ($in =~ /^[<>=!]/) {
	    Carp::cluck "DEBUG:__parse_pkg_config_line: internal oops: not cleaned: $in $raw_pkgconf[0]\n";
	    shift @raw_pkgconf;
	    next;
	}
	$storage->add_pkgconf_req($in);
    }
}

### TODO use ready M4 parsing subroutines defined below!
# use _read_any / _read_token from ::Autoconf
#grep {$_} split (/\s+/, $pkgline)
sub __read_m4_arg {
    my $lineref=shift;
    $$lineref=~s/^\s*//g;
    $$lineref=~s/^(\S*)//g;
    return $1;
}

sub __subst_vars {
    my ($in,$vars)=@_;
    return unless defined $in;
    my $line=&__strip($in);
    $line=~s/(?:^|\s)\K\$([\w_][\w_\d]*)/$vars->{$1} ? &__expand_varref($vars->{$1}) : $1/eg;
    return $line;
}

sub __expand_varref {
    my ($varref)=@_;
    # like, with_swig=yes, or with_swig=/usr/local/bin/swig
    my @VARS=grep {$_ ne 'yes' and $_ ne 'no'} @$varref;
    return join(' ',@VARS);
}

sub __strip {
    local $_=$_[0];
    Carp::cluck "undefined value" unless defined $_;
    s/\\\n//gs;
    s/^\s*//;
    s/\s*$//;
    return $_;
}

sub __split {
    my ($line)=@_;
    return grep {defined($_) and $_ ne ''} split (/\s+/, __strip($line));
}

sub __add_h {
    # not used...
    #return map {/\.h$/ ? $_ : $_.'.h'} @_;
    return @_;
}

sub __unbacktick {
    my @in=@_;
    my @out;
    foreach my $in (@in) {
	$in=~s/^\s*//;
	if ($in=~/^`([^`]+)\s+([^`]+)`\s*$/) {
	    if ($1 eq 'basename') {
		push @out, $2;
	    } else {
		# TODO; not implemented :(
		warn "DEBUG: Autoconf: __unbacktick: $in: $1 not recognized";
		push @out, $in;
	    }
	} elsif ($in=~/^`/) {
		# TODO; not implemented :(
		warn "DEBUG: Autoconf: __unbacktick: $in: not recognized";
	} else {
	    push @out, $in;
	}
    }
    return @out;
}

1;
