package RPM::Source::Dependency::Analyzer::Collector::Plugin::QMake;

use strict;
use warnings;
use Text::ParseWords qw(shellwords);

use RPM::Source::Dependency::Analyzer::Collector::Parser::QMake;
our @ISA=qw/RPM::Source::Dependency::Analyzer::Collector::Parser::QMake/;

our $verbose=0;

sub new {
    my $class=shift;
    $class->SUPER::new(
	-verbose => $verbose,
	-varhook => \&_qmake_var_handler,
	-postfilehook => \&_qmake_postfile_handler,
	@_
	);
}

sub _qmake_var_handler {
    my ($name,$args,$parser)=@_;
    my $pverbose=0;
    $pverbose||=$parser->{-verbose};
    $pverbose||=$verbose;

    my $storage=$parser->{-storage};
    print '_qmake_var_handler: ',$name,'=',$args,"\n" if $pverbose>3;
    if ($name eq 'LIBS') {
     	foreach my $word (shellwords($args)) {
	    if ($word=~s/^-l(?=\w)//) {
		$storage->add_libs_req($word);
		print STDERR 'qmake project: TODO: need lib',$word,".so\n" if $pverbose;
	    }
     	}
    } elsif ($name eq 'PKGCONFIG') {
     	foreach my $word (shellwords($args)) {
     	    $storage->add_pkgconf_req($word) if $word=~/^\w/;
     	}
    } elsif ($name eq 'TEMPLATE') {
	# if
        #TEMPLATE = lib
	# then put target to -ignore section (internal lib)?
	# or just use flag 'ban self'
	$parser->{'__TEMPLATE'}=$args;
    } elsif ($name eq 'TARGET') {
	$parser->{'__TARGET'}=$args;
    } elsif ($name eq 'QT') {
	$storage->set_lang_flag('cxx');
	# TODO proper qt3,4,5 selection
	$storage->set_lang_flag('qmake');
     	foreach my $word (shellwords($args)) {
     	    $storage->add_qt_component_req($word);
     	    print STDERR 'qmake project: need qt component ',$word,"\n" if $pverbose;
     	}
    }
}

sub _qmake_postfile_handler {
    my ($parser)=@_;
    my $template=$parser->{'__TEMPLATE'};
    my $target=$parser->{'__TARGET'};
    if ($target and $template and $template eq 'lib') {
	my $storage=$parser->{-storage};
	$storage->add_libs_ignore($target);
	my $pverbose=0;
	$pverbose||=$parser->{-verbose};
	$pverbose||=$verbose;
	print STDERR "I: ignored internal library $target\n" if $pverbose>1;
    }
}


1;
