package RPM::Source::Dependency::Analyzer::FileListSeparator;

use strict;
use warnings;

use Carp;
use File::Basename;

our $verbose=0;

# TODO

my @_KNOWN_KEYS=qw/
configureac makefileam raw_configure sconstruct makefile
qdocconf qmakepro configureoctave cmakelists
perl_buildscripts guess_perl guess_test_perl guess_scripts perl_files xs_files
pomxml
swig_files fortran_files python_files python_scripts
lua_files ruby_files go_files
c_files cpp_files texinfo
/;
my %_KNOWN_KEYS= map {$_=>1} @_KNOWN_KEYS;
our @DEFAULT_UNPACK=qw/
configureac makefileam qmakepro cmakelists
perl_buildscripts guess_perl guess_test_perl guess_scripts 
pomxml
lua_files
python_files
ruby_files
/;

sub new {
    my $class = shift;
    my $self = {
	-filetypes => {},
	-depth_prefix => '',
	-hint => '',
	#TODO ?
	#-tree_type => undef,
	# TODO: something like (rebased filelist) '^examples/'
	# already ban_filepatterns
	#-skip_analyze_by_pattern => $opt_skip_analyze_by_pattern,
	@_
    };
    bless $self, $class;

    die unless $self->{-storage};
    
    # my $hint=$self->{-hint};
    # if ($hint and $hint eq 'octave') {
    # 	$self->{-analyze_subdir_autoconf}=1 if not defined $self->{-analyze_subdir_autoconf};
    # }
    
    return $self;
}

sub add {
    my $self=shift;
    my $key=shift;
    unless ($_KNOWN_KEYS{$key}) {
	Carp::confess "Oops! internal error: unknown key $key";
    }
    push @{$self->{-filetypes}->{$key}}, @_;
}

sub has {
    my $self=shift;
    my $key=shift;
    unless ($_KNOWN_KEYS{$key}) {
	Carp::confess "Oops! internal error: unknown key $key";
    }
    my $aref=$self->{-filetypes}->{$key};
    return 1 if $aref and @$aref;
    return 0;
}

sub get {
    my $self=shift;
    my $key=shift;
    unless ($_KNOWN_KEYS{$key}) {
	Carp::confess "Oops! internal error: unknown key $key";
    }
    my $aref=$self->{-filetypes}->{$key};
    return if ! $aref;
    return @$aref;
}

my %NOT_SCRIPTS=map {$_=>1} qw/
ABOUT
API
ARTISTIC
AUTHORS
BUGS
CHANGES
Changes
ChangeLog
Changelog
CHANGELOG
configure
Copying
COPYING
copying
COPYRIGHT
copyright
crosscompile
CTAGS
ETAGS
depcomp
Dependencies
Doxyfile
FAQ
GPL
highscores
install-sh
INSTALL
LGPL
LICENSE
libtool
MACHINES
Makefile
makefile
GNUMakefile
MANIFEST
MANUAL
missing
mkinstalldirs
MPI
NEWS
PKG-INTRO
py-compile
README
README.md
requirement
THANKS
TODO
TUTORIAL
SConstruct
shtool
VERSION
/;

my %KNOWN_PERL_FINDDEP_DIR=map{$_=>1} qw/
config licences LICENCES patches
bin script scripts lib inc t dat data share src pod xs include etc templates public
doc po
/;
# utils
my %BAN_PERL_FINDDEP_DIR=map{$_=>1} qw/
benchmarks dev
eg ex example examples samples
maint
xt
/;


sub separate_filelist {
    my $self=shift;
    my $filelistref=shift;

    my $hint=$self->{-hint};
    my $storage=$self->{-storage};
    my $depth_prefix=$self->{-depth_prefix};

    my %perlroot_ban;
    my %perlroot_local_known_dir;

    # if ($hint eq 'perl') {
    # 	foreach (@$filelistref) {
    # 	    my $rebased_file=$_;
    # 	    $rebased_file=~s,^\Q$depth_prefix\E/,, if $depth_prefix;
    # 	    if ($rebased_file=~m!^([^/]+)/!) {
    # 		my $dir=$1;
    # 		if ($KNOWN_PERL_FINDDEP_DIR{$dir} or $local_known_dir{$dir}) {
    # 		    next;
    # 		} elsif ($BAN_PERL_FINDDEP_DIR{$dir}) {
    # 		    $perlroot_ban{$dir}=1;
    # 		} else {
    # 		    # TODO:
    # 		    # $ban{$dir}=1 if &__lazy_get_dialog()->msgbox( title => 'Unknown Dir', text => 'Ban '.$dir .' ?');
    # 		    $perlroot_local_known_dir{$dir}=1;
    # 		}
    # 	    }
    # 	}
    # }

    print STDERR "FileListSeparator: depth_prefix=$depth_prefix hint=$hint\n" if $verbose>1;
    
    foreach my $file (@$filelistref) {
	local $_=$file;
	my $rebased_file=$file;
	$rebased_file=~s,^\Q$depth_prefix\E/?,, if $depth_prefix;
	my $rebased_dir=dirname($rebased_file);

	print STDERR "FileListSeparator: separate_filelist: $rebased_file ( $rebased_dir )\n" if $verbose >1;

	if ($hint and ($hint eq 'perl') and $BAN_PERL_FINDDEP_DIR{$rebased_dir}) {
	    print STDERR "FileListSeparator: banned $rebased_file due to perl $rebased_dir\n" if $verbose;
	    next;
	}

	if (m!(?:^|/)debian/!) {
	    # TODO: add debian as metadata source
	    # skip
	} elsif (m!(?:^|/)configure\.(?:ac|in)$!) {
	    $self->add('configureac', $_);
	} elsif (m!(?:^|/)configure$!) {
	    $self->add('raw_configure', $_);
	} elsif (m!(?:^|/)configure\.(?:base|add)$!) {
	    $self->add('configureoctave', $_);
	} elsif (m!(?:^|/)Makefile\.am$!) {
	    $self->add('makefileam', $_);
	} elsif (m!(?:^|/)G?Makefile$!) {
	    $self->add('makefile', $_);
	} elsif (m!(?:^|/)SConstruct$!) {
	    $self->add('sconstruct', $_);
	} elsif (m!(?:^|/)(?:Makefile|Build)\.PL$!) {
	    $self->add('perl_buildscripts', $_);
	} elsif (m!(?:^|/)inc/Module/AutoInstall\.pm$!) {
	    $self->add('guess_perl', $_);
	    $storage->set_lang_flag('perl_inc_Module_AutoInstall');
	} elsif (m!(?:^|/)inc/Module/Install\.pm$!) {
	    # TODO:
	    #$self->add('guess_perl', $_);
	    #perl-Module-Install-1.06-alt1.noarch provides perl(inc/Module/Install.pm)
	    $storage->set_lang_flag('perl_inc_Module_Install');
	} elsif (m!\.pr[io]$!) {
	    $self->add('qmakepro', $_);
	} elsif (m!\.qdocconf$!) {
	    $self->add('qdocconf', $_);
	} elsif (m!(?:^|/)CMakeLists\.txt$!) {
	    $self->add('cmakelists', $_);
	} elsif (m!(?:^|/)man.*\.pod$!) {
	    $storage->set_lang_flag('prog_pod2man');
	} elsif (m!(?:^|/)(setup|__init__|.*)\.py$!) {
	    # python flag set if it is python module; should be setup.py
	    $storage->set_lang_flag('python') if $1 eq '__init__' or $1 eq 'setup';
	    $self->add('python_files', $_);
	} elsif (m!\.(?:c|h)$!) {
	    $self->add('c_files', $_);
	} elsif (m!\.(?:xs|XS)$!) {
	    $self->add('xs_files', $_);
	} elsif (m!\.i$!) {
	    $self->add('swig_files', $_);
	} elsif (m!\.(?:f)$!) {
	    $self->add('fortran_files', $_);
	} elsif (m!\.(?:C|CPP|CXX|c(?:c|pp|xx))$!) {
	    # cc|cxx|C|cpp
	    $self->add('cpp_files', $_);
	} elsif (m!\.(?:go)$!) {
	    $self->add('go_files', $_);
	} elsif (m!\.(?:rb)$!) {
	    $self->add('ruby_files', $_);
	} elsif (m!\.(?:lua)$!) {
	    $self->add('lua_files', $_);
    # TODO: file' magic, skip if unpack
	} elsif (m!\.p[lm]$!) {
	    $self->add('guess_perl', $_);
	} elsif (m!\.p[lm]\.PL$!) {
	    $self->add('guess_perl', $_);
	} elsif (m!(?:^|/)t/.+\.(?:t|pm)$!) {
	    $self->add('guess_test_perl', $_);
	} elsif (m!\.f?cgi$!) {
	    $self->add('guess_scripts', $_);
	# files in root w/o extension
	} elsif (m!(?:^|/)[^./]+$!) {
	    $self->add('guess_scripts', $_) unless $NOT_SCRIPTS{basename($_)};
	} elsif (m!(?:^|/)pom\.xml$!) {
	    $self->add('pomxml', $_);
	} elsif (m!\.texi$!) {
	    $self->add('texinfo', $_);
	}
    }
}

1;

__END__

sub TODO_make_an_option_check_dirs {
    my $filelist=shift;
    my %ban;
    my %local_known_dir;
    foreach (@$filelist) {
	if (m!^([^/]+)/!) {
	    my $dir=$1;
	    if ($KNOWN_FINDDEP_DIR{$dir} or $local_known_dir{$dir}) {
		next;
	    } elsif ($BAN_FINDDEP_DIR{$dir}) {
		$ban{$dir.'/'}=1;
	    } else {
		# TODO:
		$ban{$dir.'/'}=1 if &__lazy_get_dialog()->msgbox( title => 'Unknown Dir', text => 'Ban '.$dir .' ?');
		$local_known_dir{$dir}=1;
	    }
	}
    }
    return \%ban;
}

sub __lazy_get_dialog {
    CORE::state $ui_dialog;
    return $ui_dialog if $ui_dialog;
    require UI::Dialog;
    $ui_dialog = new UI::Dialog ( backtitle => 'PerlBuilder', title => 'Default',);
#			 height => 20, width => 65 , listheight => 5,
#			 order => [ 'zenity', 'xdialog' ] );
    return $ui_dialog;
}

1;

