package RPM::Source::Dependency::Analyzer::Resolver::QMakeResolver;

use strict;
use warnings;
use Carp;

use RPM::Source::Dependency::Analyzer::Resolver::SingleType;
our @ISA=qw/RPM::Source::Dependency::Analyzer::Resolver::SingleType/;

our $verbose=0;

# note: testlib map is added manually, see pri files
my %qt4lib=qw/
	testlib	QtTest

	3support	Qt3Support
	av	QtAV
	avwidgets	QtAVWidgets
	assistantclient	QtAssistantClient
	bearer	QtBearer
	clucene	QtCLucene
	connectivity	QtConnectivity
	contacts	QtContacts
	core	QtCore
	dbus	QtDBus
	declarative	QtDeclarative
	designer	QtDesigner
	designercomponents	QtDesignerComponents
	feedback	QtFeedback
	glib	QtGLib
	glib-2.0	QtGLib-2.0
	gallery	QtGallery
	gui	QtGui
	help	QtHelp
	location	QtLocation
	mimetypes	QtMimeTypes
	multimedia	QtMultimedia
	multimediakit	QtMultimediaKit
	network	QtNetwork
	opengl	QtOpenGL
	organizer	QtOrganizer
	publishsubscribe	QtPublishSubscribe
	script	QtScript
	scripttools	QtScriptTools
	sensors	QtSensors
	serialport	QtSerialPort
	serviceframework	QtServiceFramework
	sql	QtSql
	svg	QtSvg
	systeminfo	QtSystemInfo
	uitools	QtUiTools
	versit	QtVersit
	versitorganizer	QtVersitOrganizer
	webkit	QtWebKit
	xml	QtXml
	xmlpatterns	QtXmlPatterns
/;
my %qt5lib=qw/
	testlib	Qt5Test

	3danimation	Qt53DAnimation
	3dcore	Qt53DCore
	3dextras	Qt53DExtras
	3dinput	Qt53DInput
	3dlogic	Qt53DLogic
	3dquick	Qt53DQuick
	3dquickanimation	Qt53DQuickAnimation
	3dquickextras	Qt53DQuickExtras
	3dquickinput	Qt53DQuickInput
	3dquickrender	Qt53DQuickRender
	3dquickscene2d	Qt53DQuickScene2D
	3drender	Qt53DRender
	bluetooth	Qt5Bluetooth
	charts	Qt5Charts
	concurrent	Qt5Concurrent
	core	Qt5Core
	dbus	Qt5DBus
	datavisualization	Qt5DataVisualization
	designer	Qt5Designer
	designercomponents	Qt5DesignerComponents
	eglfsdeviceintegration	Qt5EglFSDeviceIntegration
	eglfskmssupport	Qt5EglFsKmsSupport
	glib-2.0	Qt5GLib-2.0
	gstreamer-1.0	Qt5GStreamer-1.0
	gstreamerquick-1.0	Qt5GStreamerQuick-1.0
	gstreamerui-1.0	Qt5GStreamerUi-1.0
	gstreamerutils-1.0	Qt5GStreamerUtils-1.0
	gamepad	Qt5Gamepad
	gui	Qt5Gui
	help	Qt5Help
	hunspellinputmethod	Qt5HunspellInputMethod
	location	Qt5Location
	multimedia	Qt5Multimedia
	multimediagsttools	Qt5MultimediaGstTools
	multimediaquick	Qt5MultimediaQuick
	multimediawidgets	Qt5MultimediaWidgets
	network	Qt5Network
	networkauth	Qt5NetworkAuth
	nfc	Qt5Nfc
	opengl	Qt5OpenGL
	pas	Qt5Pas
	positioning	Qt5Positioning
	positioningquick	Qt5PositioningQuick
	printsupport	Qt5PrintSupport
	qml	Qt5Qml
	quick	Qt5Quick
	quickcontrols2	Qt5QuickControls2
	quickparticles	Qt5QuickParticles
	quickshapes	Qt5QuickShapes
	quicktemplates2	Qt5QuickTemplates2
	quicktest	Qt5QuickTest
	quickwidgets	Qt5QuickWidgets
	remoteobjects	Qt5RemoteObjects
	script	Qt5Script
	scripttools	Qt5ScriptTools
	scxml	Qt5Scxml
	sensors	Qt5Sensors
	serialbus	Qt5SerialBus
	serialport	Qt5SerialPort
	sql	Qt5Sql
	svg	Qt5Svg
	texttospeech	Qt5TextToSpeech
	virtualkeyboard	Qt5VirtualKeyboard
	waylandclient	Qt5WaylandClient
	waylandcompositor	Qt5WaylandCompositor
	webchannel	Qt5WebChannel
	webengine	Qt5WebEngine
	webenginecore	Qt5WebEngineCore
	webenginewidgets	Qt5WebEngineWidgets
	webkit	Qt5WebKit
	webkitwidgets	Qt5WebKitWidgets
	websockets	Qt5WebSockets
	webview	Qt5WebView
	widgets	Qt5Widgets
	x11extras	Qt5X11Extras
	xcbqpa	Qt5XcbQpa
	xdg	Qt5Xdg
	xdgiconloader	Qt5XdgIconLoader
	xml	Qt5Xml
	xmlpatterns	Qt5XmlPatterns
/;

# STATIC method
# TODO: arg: qt_context storage, checked against qt4 and 5
sub find_applicable_qt_contexts {
    ...
}

sub _map_and_resolve {
    my ($self,$candidate)=@_;
    my $qt_context=$self->{-target};
    warn 'Resolver::QMakeResolver: qt=', $qt_context,' candidate=',$candidate, "\n" if $verbose;
    return [], undef unless $qt_context;
    my $libname;
    if ($qt_context == 5) {
	$libname=$qt5lib{$candidate};
    } elsif ($qt_context == 4) {
	$libname=$qt4lib{$candidate};
    } else {
	warn 'WARNING: Resolver::QMakeResolver: unsupported qt ', $qt_context, "\n";
	return [], undef;
    }
    return [], undef unless $libname;
    warn 'Resolver::QMakeResolver: found lib: ',$libname,"\n" if $verbose;
    my @data=$self->_distrodb_translate('devel-libs',$libname);
    #if @data
    return \@data, undef;
}




1;

__END__
#data generated with
#grep -rh ^Qt5 ~/.cache/distrodb/altlinux/sisyphus/devel-libs |
while (<>) {
	chomp;
	/^(\S+)/;
	my $lib=$1;
	my $qt=lc($lib=~s/^Qt5?//r);
	print "\t$qt\t$lib\n";

}
