package RPM::Source::Dependency::Analyzer::Resolver::SingleType;

use strict;
use warnings;
use Carp;

our $verbose=0;

sub new {
    my $class=shift;
    my $self={
	-map_always =>{},
	@_,
    };
    return bless $self,$class;
}

sub get_buildreq {
    my $self=shift;
    return $self->{-buildreqobj};
}

sub get_oldauto_buildreq {
    my $self=shift;
    return $self->{-buildreq_oldauto_obj};
}

sub get_resolver {
    my $self=shift;
    return $self->{-resolverobj};
}

sub get_storage {
    my $self=shift;
    return $self->{-storageobj};
}

#sub get_distroinfo {
#    my $self=shift;
#    return $self->{-distroinfoobj};
#}

sub get_ignore_hash {
    my $self=shift;
    return $self->{-IGNORE_NOT_FOUND_hash};
}

sub get_input_drop_hash {
    my $self=shift;
    return $self->{-DROP_hash};
}

sub get_distrodbname {
    my $self=shift;
    return $self->{-distrodb_name};
}

sub get_storage_name {
    my $self=shift;
    my $storage_name=$self->{-storage_name};
    unless ($storage_name) {
	my $distrodb_name=$self->{-distrodb_name};
	confess 'internal error: no storage key and no distrodb name for ',$self->label unless $distrodb_name;
	$storage_name=$self->get_storage->distrodb_name2storage_name($distrodb_name);
	die q!internal error: can't resolve storage key for !,$self->label unless $storage_name;
	$self->{-storage_name}=$storage_name;
    }
    return $storage_name;
}

sub label {
    my $self=shift;
    my $name=$self->{-label};
    $name //= $self->{-distrodb_name};
    $name //= $self->{-storage_name};
    return $name;
}

sub is_already_present_pkg {
    my $self=shift;
    my $pkg=shift;
    return $self->{-already_present_pkgs}->{$pkg};
}

# sub get_collector_for_br {
#     my $self=shift;
#     return $self->{-resolved_br_ref};
# }

# sub get_collector_for_ignore {
#     my $self=shift;
#     return $self->{-resolved_ignore_ref};
# }

sub push2ignore_collector {
    my $self=shift;
    map {Carp::cluck "undefined value" unless defined $_} @_;
    push @{$self->{-resolved_ignore_ref}}, @_;
}

sub push2br_collector {
    my $self=shift;
    map {Carp::cluck "undefined value" unless defined $_} @_;
    push @{$self->{-resolved_br_ref}}, @_;
    if ($verbose>1) {
	print STDERR 'DEBUG: push2br_collector(',$self->label,'): ',join(' ',@_),"\n";
    }
}

sub push2mapped_collector {
    my $self=shift;
    map {Carp::cluck "undefined value" unless defined $_} @_;
    push @{$self->{-mapped_br_ref}}, @_;
    if ($verbose>1) {
	print STDERR 'DEBUG: push2mapped_collector(',$self->label,'): ',join(' ',@_),"\n";
    }
}

sub set_flag_not_found {
    my $self=shift;
    ${$self->{-flag_not_found_ref}}=1;
}

sub is_mappable {
    my $self=shift;
    return $self->{-is_mappable};
}

sub get_mappable_optsuffix {
    my $self=shift;
    return $self->{-mappable_optsuffix};
}

sub map2br {
    my $self=shift;
    my $mapper=$self->{-mapper};
    my @mapped=map {&$mapper($_)} @_;
    return wantarray ? @mapped:$mapped[0];
}

########################################
#                                      #
#          M A I N   C A L L           #
#                                      #
########################################

sub resolve {
    my ($self)=@_;
    my $storage=$self->get_storage;
    my $storage_key=$self->get_storage_name;
    my $db_ignore_list=$storage->getref_raw_storage_ignore($storage_key);
    my $db_req_list=$storage->getref_raw_storage_req($storage_key);

    map {$self->_resolve_ignore($_)} @$db_ignore_list;

    my %storage_drop=map {$_=>1} @$db_ignore_list;
    map {$self->_resolve_br($_,\%storage_drop)} @$db_req_list;
}

sub _resolve_ignore {
    my ($self,$name)=@_;

    $name=$self->_prefilter($name);
    return unless defined $name;
    my ($dataref, $mapped_name, $resolved_name)=$self->_pretranslate($name);
    $self->push2ignore_collector( @$dataref ) if $dataref;
    $self->push2ignore_collector( $mapped_name ) if $mapped_name;
}

sub _resolve_br {
    my ($self,$name,$storage_drop)=@_;

    $name=$self->_prefilter($name,$storage_drop);
    return unless defined $name;

    my ($dataref, $mapped_name, $resolved_name)=$self->_pretranslate($name);

    return if $self->_is_already_known_dependency($dataref,$mapped_name);
    ### used in PathResolver ###
    $self->_postprocess($dataref,\$mapped_name);
    $self->_process_dups($dataref, \$mapped_name, $name, $resolved_name);
    $self->_warn_if_not_found($dataref,$name);

    if (
	defined($mapped_name) and not $self->__is_resolve_mapped
	and ((@$dataref and not exists $self->{-map_always}->{$dataref->[0]}) or (!@$dataref and $self->__is_allow_unknown_mapped))
	) {
	print STDERR 'DEBUG: _resolve_br: push2mapped_collector: mapped_name=',$mapped_name,' resolved=',join(',',@$dataref),"\n" if $verbose>1;
	$self->push2mapped_collector( [$mapped_name,$dataref] );
    } else {
	$self->push2br_collector( @$dataref );
    }
}

sub __is_resolve_mapped {
    my $self=shift;
    my $retval=$self->{__is_resolve_mapped};
    return $retval if defined $retval;
    # lazy calculation
    my $resolver=$self->get_resolver;
    my $optsuffix=$self->get_mappable_optsuffix;
    print STDERR "Oops! is_resolve: no optsuffix for ",$self->label,"\n" unless defined $optsuffix;
    #print STDERR "mapped name=".$mapped_name,"\n" unless defined $optsuffix;
    $retval=$resolver->{-resolve_.$optsuffix} ? 1:0;
    $self->{__is_resolve_mapped}=$retval;
    return $retval;
}

sub __is_allow_unknown_mapped {
    my $self=shift;
    my $resolver=$self->get_resolver;
    my $optsuffix=$self->get_mappable_optsuffix;
    print STDERR "Oops! is_allow_unknown: no optsuffix for ",$self->label,"\n" unless defined $optsuffix;
    return $resolver->{-allow_unknown_.$optsuffix};
}

sub _prefilter {
    my ($self,$name,$storage_drop)=@_;
    my $input_drop=$self->get_input_drop_hash;
    my @valid_names;
    my @names=($name);
    @names=@$name if ref($name) eq 'ARRAY';
    foreach my $candidate (@names) {
	print STDERR 'SourceAnalyzer:SingleType(',$self->label,') checking: ',$candidate,"\n" if $verbose;
	if ($input_drop and $input_drop->{$candidate}) {
	    print STDERR 'SourceAnalyzer:SingleType(',$self->label,') dropped (input): ',$candidate,"\n" if $verbose;
	    next;
	}
	if ($storage_drop and $storage_drop->{$candidate}) {
	    print STDERR 'SourceAnalyzer:SingleType(',$self->label,') dropped (storage): ',$candidate,"\n" if $verbose;
	    next;
	}
	push @valid_names, $candidate;
    }
    if (not @valid_names) {
	return;
    } elsif (ref $name) {
	return \@valid_names;
    } else {
	return $valid_names[0];
    }
}

sub _pretranslate {
    my ($self,$name)=@_;

    my ($dataref,$mapped_name,$resolved_name);
    my @names=($name);
    @names=@$name if ref($name) eq 'ARRAY';
    warn 'Oops! impossible happened -- names are not found' unless @names;
    foreach my $candidate (@names) {
	($dataref,$mapped_name) = $self->_map_and_resolve($candidate);
	if (@$dataref>0) {
	    $resolved_name=$candidate;
	    last;
	}
    }
    return $dataref, $mapped_name, $resolved_name;
}

sub _map_and_resolve {
    my ($self,$candidate)=@_;
    my ($mapped_name,@data);
    $mapped_name=$self->map2br($candidate) if $self->is_mappable;
    @data=$self->_distrodb_translate($self->get_distrodbname,$candidate);
    return \@data, $mapped_name;
}

sub _distrodb_translate {
    my ($self,$db,$candidate)=@_;
    my $distroinfo=$self->{-distroinfoobj};
    my @data=$distroinfo->translate($db,$candidate);
    if (@data>1) {
	# filter multiple using list.src.shadow
	my @newdata;
	foreach my $mapped (@data) {
	    my $srcname=$distroinfo->translate('sourcename',$mapped);
	    push @newdata, $mapped if
		not $srcname or not $distroinfo->exist('list.src.shadow',$srcname);
	};
	print STDERR "_distrodb_translate: conflict in $db for $candidate: filtered ",join(',',@data)," -> ",join(',',@newdata),"\n" if $verbose;
	@data=@newdata;
    }
    return @data;
}


sub _is_already_known_dependency {
    my ($self,$dataref,$mapped_name)=@_;
    my $resolver=$self->get_resolver;
    my $optsuffix=$self->get_mappable_optsuffix;
	return 1 if defined($mapped_name) and $self->__buildreq_have_this($mapped_name)
	    or @$dataref and (
		$self->__buildreq_have_any($dataref) and not 
		(defined($mapped_name) and $resolver->{-keep_explicit_.$optsuffix})
	    );
    return 0;
}

sub _postprocess {}

# name is a choice like [lua5.1 lua5 lua]
# $resolved_name is what name in a choice we actually found in db
# mapped_name is like pkgconfig(lua5)
# dataref points to packages: [lua51-devel lua-devel]
sub _process_dups {
    my ($self, $dataref, $mapped_nameref, $name, $resolved_name)=@_;
    my @names=ref($name) ? @$name : $name;
    my $mapped_name=$$mapped_nameref;
    print STDERR 'DEBUG: _process_dups(',$self->label,'): data=', join(';',@$dataref), ' names are [', join(' ',@names), ']',(defined($resolved_name) ? " resolved name=$resolved_name":''),(defined($mapped_name) ? " mapped name=$mapped_name":''), "\n" if $verbose > 1;

    my $oldauto_buildreq=$self->get_oldauto_buildreq;
    if ($oldauto_buildreq and @$dataref>1) {
	foreach my $pkg (@$dataref) {
	    # TODO: check unique reverse provides of $pkg here
	    if ($oldauto_buildreq->has_buildreq($pkg) or $oldauto_buildreq->has_pre_buildreq($pkg)) {
		@$dataref=$pkg;
		last;
	    }
	}
    }

    if (@$dataref>1) {
	my $resolver=$self->get_resolver();
	foreach my $resolution_hash (
	    \%RPM::Source::Dependency::Analyzer::Resolver::opt_select_multiple_dependency,
	    $resolver->{-select_multiple_dependency}) {
	    my $resolution;
	    $resolution=$resolution_hash->{$mapped_name} if defined $mapped_name;
	    $resolution=$resolution_hash->{$resolved_name} unless defined $resolution;
	    if (defined $resolution) {
		@$dataref=$resolution;
		# mapped name is used as replacer of is_mappable flag -- undef should be kept
		$$mapped_nameref=$dataref->[0] if defined $mapped_name;
		print STDERR 'DEBUG: _process_dups(',$self->label,'): forced ',$resolved_name,' to ',$dataref->[0],"\n" if $verbose >1;
		last;
	    }
	}

	my $is_override_func=$resolver->{-package_override_func};
	if (@$dataref>1 and $is_override_func) {
	    my $stateobj=$resolver->{-package_override_stateobj};
	    my @overriders=@$dataref;
	    my @newdata=@$dataref;
	    foreach my $overrider (@overriders) {
		@newdata=grep {!&$is_override_func($stateobj,$overrider,$_)} @newdata;
	    }
	    @$dataref=@newdata;
	    if (1==@newdata) {
		# mapped name is used as replacer of is_mappable flag -- undef should be kept
		$$mapped_nameref=$dataref->[0] if defined $mapped_name;
		print STDERR 'DEBUG: _process_dups2(',$self->label,'): forced ',$resolved_name,' to ',$dataref->[0],"\n" if $verbose >1;
	    }
	}

	if (@$dataref>1 and ($verbose or $self->{-verbose})) {
	    warn 'WARNING: SourceAnalyzer: multiple provides in ',$self->label,' for ',join(' ',@names),
		(defined($mapped_name) ? " [$mapped_name]": ''),': ',join(' ',@$dataref),"\n";
	}
    }
}

sub __hack_no_warning_for_name_used_only_once_DROPME {
    return \%RPM::Source::Dependency::Analyzer::Resolver::opt_select_multiple_dependency;
}

sub _warn_if_not_found {
    my ($self, $dataref, $name)=@_;
    my @names=ref($name) ? @$name : $name;
    if (not @$dataref and $self->__names_are_not_ignored(@names)) {
	$self->set_flag_not_found;
	print STDERR 'INFO: SourceAnalyzer: nothing in ', $self->label, ' provides ', join(' ',@names), ".\n" if $verbose or $self->{-verbose};
    }
}

sub __names_are_not_ignored {
    my ($self,@names)=@_;
    my $ignore=$self->get_ignore_hash;
    return 1 if not $ignore;
    foreach my $name (@names) {
	return 1 if not $ignore->{$name};
    }
    return 0;
}


sub __buildreq_have_any {
    my ($self,$list)=@_;
    my $buildreq=$self->get_buildreq;
    foreach my $key (@$list) {
	Carp::cluck "__buildreq_have_any: got undef" unless defined $key;
	return 1 if $buildreq->has_buildreq($key);
	return 1 if $buildreq->has_pre_buildreq($key);
	return 1 if $buildreq->has_ignore_buildreq($key);
	return 1 if $self->is_already_present_pkg($key);
    }
    return 0;
}

sub __buildreq_have_this {
    my ($self,$key)=@_;
    Carp::cluck "__buildreq_have_this: got undef" unless defined $key;
    my $buildreq=$self->get_buildreq;
    return 1 if $buildreq->has_buildreq($key);
    return 1 if $buildreq->has_pre_buildreq($key);
    return 1 if $buildreq->has_ignore_buildreq($key);
    return 1 if $self->is_already_present_pkg($key);
    return 0;
}



1;

__END__
