package RPM::Source::Dependency::Analyzer::Resolver::SingleTypeFactory;

use strict;
use warnings;
use Carp;

use RPM::Source::Dependency::Analyzer::Resolver::SingleType;
use RPM::Source::Dependency::Analyzer::Resolver::PathResolver;
use RPM::Source::Dependency::Analyzer::Resolver::CMakeResolver;
use RPM::Source::Dependency::Analyzer::Resolver::QMakeResolver;
use RPM::Source::Dependency::Analyzer::Resolver::PythonResolver;

sub new {
    my $class=shift;
    my $self={
	@_,
    };

    foreach my $key (qw/
        -resolverobj
        -buildreqobj
        -distroinfoobj
        -storageobj
        -flag_not_found_ref
        -resolved_br_ref
        -mapped_br_ref
        -resolved_ignore_ref
     /) {
	die "Resolver::SingleTypeFactory: $key is not initialized" unless $self->{$key};
    }

    foreach my $key (qw/
        -buildreq_oldauto_obj
     /) {
	die "Resolver::SingleTypeFactory: $key does not exist" unless exists $self->{$key};
    }


    my $buildreq=$self->{-buildreqobj};
    my $distroinfo=$self->{-distroinfoobj};
    my %uniquely_provided_packages;
    foreach my $known_provided (
	$buildreq->get_list_buildreq(),
	$buildreq->get_list_pre_buildreq(),
	$buildreq->get_list_ignore_buildreq(),
	) {
	my @pkgs=$distroinfo->translate('provides',$known_provided);
	if (@pkgs==1) {
	    $uniquely_provided_packages{$pkgs[0]}=1;
	}
    }
    $self->{-already_present_pkgs} = \%uniquely_provided_packages;

    return bless $self,$class;
}

sub create {
    my $self=shift;
    my %args=(
	-DROP_hash => {},
	-mapper => sub { return $_[0]},
	-is_mappable => 0,
	@_,
	);

    my $class=$args{-class};
    # dbname is not mandatory fore custom class
    if ($class) {
	die "Resolver::SingleTypeFactory: storage or distrodb name is not initialized" if not $args{-distrodb_name} and not $args{-storage_name};
    } else {
	die "Resolver::SingleTypeFactory: -distrodb_name is not initialized" if not $args{-distrodb_name};
    }
    if ($args{-is_mappable}) {
	foreach my $key (qw/
        -mapper
        -mappable_optsuffix
     /) {
	    die "Resolver::SingleTypeFactory: $key is not initialized" unless $args{$key};
	}
    }

    my @inst_args=(
	# passed as args
	-verbose => $self->{-verbose},
	-resolverobj => $self->{-resolverobj},
	-buildreqobj => $self->{-buildreqobj},
        -buildreq_oldauto_obj => $self->{-buildreq_oldauto_obj},
	-distroinfoobj => $self->{-distroinfoobj},
	-storageobj => $self->{-storageobj},
	-flag_not_found_ref => $self->{-flag_not_found_ref},
	-resolved_br_ref => $self->{-resolved_br_ref},
	-mapped_br_ref => $self->{-mapped_br_ref},
	-resolved_ignore_ref => $self->{-resolved_ignore_ref},
	# created by factory
	-already_present_pkgs => $self->{-already_present_pkgs},

	# per instance args
	-label => $args{-label},
	-storage_name => $args{-storage_name},
	-distrodb_name => $args{-distrodb_name},
	-IGNORE_NOT_FOUND_hash => $args{-IGNORE_NOT_FOUND_hash},
	-DROP_hash => $args{-DROP_hash},
        -is_mappable => $args{-is_mappable},
        -mapper => $args{-mapper},
        -mappable_optsuffix => $args{-mappable_optsuffix},
	# arg; used in python, qmake
        -target => $args{-target},
	);
    push @inst_args, -map_always => $args{-map_always} if $args{-map_always};
    $class //= 'RPM::Source::Dependency::Analyzer::Resolver::SingleType';

    return $class->new(@inst_args);
}

1;

__END__
