package RPM::Source::Dependency::HackOverSourceAnalyzer;

use strict;
use warnings;
our $verbose=0;

my $select_multiple_dependency_db = {
	    '/usr/bin/c++' => 'gcc-c++',
	    # '/usr/bin/dvips' => 'texlive-base-bin',
	    # '/usr/bin/latex' => 'texlive-latex-base',
	    # '/usr/bin/pslatex' => 'texlive-latex-base',
	    # '/usr/bin/epstopdf' => 'texlive-extra-utils',
	    # '/usr/bin/texi2html' => 'texi2html',
	    '/usr/bin/java' => 'java-devel-default',
	    '/usr/bin/javac' => 'java-devel-default',
	    '/usr/bin/javadoc' => 'java-devel-default',
	    '/usr/bin/javah' => 'java-devel-default',
	    '/usr/bin/javap' => 'java-devel-default',
	    '/usr/bin/jar' => 'java-devel-default',
	    '/usr/bin/flex' => 'flex',
	    '/usr/bin/flex++' => 'flex',
	    '/usr/bin/rsync' => 'rsync',
#	    '/usr/bin/wx-config' => 'libwxGTK-devel',
	    'ixp' => 'libixp-devel',
	    'net' => 'libnet2-devel',
	    'libnet.h' => 'libnet2-devel',
	    'netlink' => 'libnetlink-devel',
	    'libpng' => 'libpng-devel',
	    'png' => 'libpng-devel',
	    'png.h' => 'libpng-devel',
	    'raw1394' => 'libraw1394-devel',
	    'libraw1394/raw1394.h' => 'libraw1394-devel',
	    'glut' => 'libfreeglut-devel',
	    'GL/glut.h' => 'libfreeglut-devel',
	    'xercesc/util/XercesDefs.hpp' => 'libxerces-c-devel',
	    'xercesc/util/PlatformUtils.hpp' => 'libxerces-c-devel',
	    'xercesc/dom/DOM.hpp' => 'libxerces-c-devel',
	    'xerces-c' => 'libxerces-c-devel',
	    'cpufreq.h' => 'libcpufreq-devel',
	    'db-4.7' => 'libdb4.8-devel',
	    'db-4.8' => 'libdb4.8-devel',
	    'db4/db.h' => 'libdb4.8-devel',
	    'db_185.h' => 'libdb4.8-devel',
	    'db' => 'libdb4.8-devel',
	    'db_cxx' => 'libdb4.8_cxx-devel',
	    'db.h' => 'libdb4.8-devel',
	    'Xm/Xm.h' => 'libopenmotif-devel',
	    'Xm' => 'libopenmotif-devel', # lib
	    'ltdl.h' => 'libltdl-devel', #libltdl7-devel
	    'fftw3' => 'libfftw3-devel', # pkg-config
	    'readline/readline.h' => 'libreadline-devel',
	    'readline' => 'libreadline-devel',
	    'readline/history.h' => 'libreadline-devel',
	    #OpenEXR/half.h: ilmbase1.0-devel ilmbase11-devel ilmbase-devel
	    'OpenEXR/half.h' => 'ilmbase-devel',
	    'crypto' => 'libssl-devel',
	    'ssl' => 'libssl-devel',
	    'openssl/conf.h' => 'libssl-devel',
	    'openssl/evp.h' => 'libssl-devel',
	    'openssl/objects.h' => 'libssl-devel',
	    # pkgconfig
	    'libcrypto' => 'libssl-devel',
	    'libssl' => 'libssl-devel',
	    'openssl' => 'libssl-devel',
};

my @pkg_override=qw/
libssl-devel	LibreSSL-devel
libatlas-devel	libclapack-devel
libltdl7-devel	libltdl3-devel
libwxGTK3.0-devel	wxGTK-devel
/;

my $pkg_override={};
while (@pkg_override>0) {
    my $big=shift @pkg_override;
    my $small=shift @pkg_override;
    $pkg_override->{$big}->{$small}=1;
}

sub package_override {
    my (undef,$big,$small)=@_;
    return $pkg_override->{$big}->{$small};
}

sub get_source_analyzer_hack_opts {
    return -select_multiple_dependency => $select_multiple_dependency_db,
	-package_override_func => \&package_override;
}

1;

# TODO
__END__
devel-libs for arpack: libarpack-ng-devel libarpack-devel
devel-libs for blas libblas blas_win32_MT blas_win32: libblas-devel libatlas-devel
devel-libs for blas: libblas-devel libatlas-devel
devel-libs for glpk: libglpk-devel libglpk36-devel libglpk-cut-log-devel
devel-libs for glut freeglut: libGLUT-devel libfreeglut-devel
devel-libs for gslcblas: libgsl90-devel libgsl-devel
devel-libs for gsl: libgsl90-devel libgsl-devel
devel-libs for gsl: libgsl90-devel libgsl-devel
devel-libs for gsl: libgsl90-devel libgsl-devel
devel-libs for gtest: libtrilinos10-devel libgtest-devel
devel-libs for Half: ilmbase1.0-devel ilmbase11-devel ilmbase-devel
devel-libs for history: libreadline5-devel libreadline-devel
devel-libs for Iex: ilmbase1.0-devel ilmbase11-devel ilmbase-devel
devel-libs for IlmThread: ilmbase1.0-devel ilmbase11-devel ilmbase-devel
devel-libs for Imath: ilmbase1.0-devel ilmbase11-devel ilmbase-devel
devel-libs for ltdl: libltdl3-devel libltdl7-devel
devel-libs for ltdl: libltdl3-devel libltdl7-devel
devel-libs for qwt: libqwt-devel libqwt6-devel
devel-libs for $withval db: libdb4.8-devel libdb4.7-devel libdb6.1-devel
devel-libs for yajl: libyajl1-devel libyajl-devel
devel-libs for yajl: libyajl1-devel libyajl-devel
headers for gsl/gsl_sf_gamma.h: libgsl90-devel libgsl-devel
headers for gtest/gtest.h: trilinos10-headers libgtest-devel
path for /usr/bin/bibtex: tetex-core texlive-base-bin
path for /usr/bin/convert: ImageMagick-tools GraphicsMagick-ImageMagick-compat
path for /usr/bin/dvipdfm: tetex-core texlive-base-bin
path for /usr/bin/emacs: emacs24-X11-athena emacs24-X11-gtk emacs24-X11-gtk3 emacs24-X11-motif emacs24-nox
path for /usr/bin/lpr: foomatic-db-engine cups
path for /usr/bin/makeindex: tetex-core texlive-base-bin
path for /usr/bin/mcs: mono-mcs mono4-core
path for /usr/bin/mono: mono mono4-core
path for /usr/bin/mpost: tetex-core texlive-metapost
path for /usr/bin/mysql: MySQL-client mariadb-client
path for /usr/bin/pdflatex: texlive-latex-base tetex-latex
path for /usr/bin/sphinx-build: python3-module-sphinx-sphinx-build-symlink python-module-sphinx
path for /usr/bin/unix2dos: unix2dos hd2u dos2unix
path for /usr/bin/xdvi: tetex-xdvi texlive-extra-utils
pkg-config for mono-cairo: mono-devel mono4-devel
pkg-config for mono: mono-devel mono4-devel
