#!/usr/bin/perl

use strict;
use warnings;

use Cwd;
use File::Temp qw/tempfile tempdir/;
use File::Path qw(make_path remove_tree);
use File::Spec;
use File::Basename;

use Data::Dumper;
use DistroMap 0.05;
use RPM::Source::Editor 0.900;
use RPM::Source::Tools::PackageFilter;
use RPM::Source::Tools::BuildReqStorage;
use Source::Bundle::Factory;
use RPM::Source::Dependency::Analyzer;
my $distro_sisyphus=DistroMap::Repository->new(-repo=>'altlinux', -branch=>'sisyphus');
my $verbose=2;
#$RPM::Source::Dependency::Analyzer::Collector::Plugin::CMake::verbose=2;
#$RPM::Source::Dependency::Analyzer::Collector::Plugin::AutoMake::verbose=9;
$Source::Shared::verbose=1;

#print Dumper($distro_sisyphus);

map {&analyze($_)} @ARGV;

sub analyze {
    my ($file)=@_;
    my $buildreq=RPM::Source::Tools::BuildReqStorage->new();
    my $tarball=Source::Bundle::Factory->create($file);
    my $storage=RPM::Source::Dependency::Analyzer::Storage->new();
    my $collector=RPM::Source::Dependency::Analyzer::Collector->new();
    $collector->collect_source_dependencies($storage,$tarball);
    my $resolver=RPM::Source::Dependency::Analyzer::Resolver->new(
	-verbose => $Source::Shared::verbose,
	-distroinfo => $distro_sisyphus,
	);
    print STDERR 'arch detected.'."\n" if $resolver->is_arch_dependent($storage);
    $storage->dump;
}


1;


__END__
