package RPM::Source::Transformation::Factory::DependencyAnalyzer::CollectorFlags;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Resource::DependencyAnalyzer::Storage;
use RPM::Source::Transformation::Factory;

our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=1;

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('collector/add_libs.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('collector/add_pkgconfig.txt',FLAG_RESOURCE_LIST_AS_ARRAY);

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRECONVERT,
	-name => 'sourcedep_collector_flags',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my (undef,undef,undef,$opt,$resourceobj)=@_;

    my $storage=$resourceobj->dependency_analyzer_storage;
    my $add_libs=$resourceobj->get_flagdb('collector/add_libs.txt');
    my $add_pkgconf=$resourceobj->get_flagdb('collector/add_pkgconfig.txt');

    $storage->add_libs_req( @$add_libs ) if $add_libs;
    $storage->add_pkgconf_req( @$add_pkgconf ) if $add_pkgconf;

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
