package RPM::Source::Transformation::Resource::DependencyAnalyzer::IsArchDependent;

use strict;
use warnings;
use Carp;
use Data::Dumper;

use RPM::Source::Dependency::Analyzer::Resolver;
use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Resource::DependencyAnalyzer::Storage;
use RPM::Source::Transformation::Resource;
our @ISA=qw/RPM::Source::Transformation::Resource/;

our $verbose=0;

push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('known-noarch.txt',FLAG_RESOURCE_LIST_AS_FLAG);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('known-arch.txt',FLAG_RESOURCE_LIST_AS_FLAG);

package Source::Shared::ResourceContainer;

sub dependency_analyzer_is_arch_dependent {
    my ($resourceobj)=@_;
    my $storage=$resourceobj->{'__DependencyAnalyzerStorage'};
    Carp::croak "Oops! dependency_analyzer_is_arch: no storage" if ! $storage;

    my $known_arch=$resourceobj->get_flagdb('known-arch.txt');
    my $known_noarch=$resourceobj->get_flagdb('known-noarch.txt');
    if ($verbose) {
	if ($known_arch || $known_noarch) {
	    print STDERR "DEBUG: dependency_analyzer_is_arch_dependent: use flag db: known_noarch\n" if $known_noarch;
	    print STDERR "DEBUG: dependency_analyzer_is_arch_dependent: use flag db: known_arch\n" if $known_arch;
	} else {
	    print STDERR "DEBUG: dependency_analyzer_is_arch_dependent: flag db: none\n";
	}
    }
    return 0 if $known_noarch;
    return 1 if $known_arch;
    return RPM::Source::Dependency::Analyzer::Resolver->is_arch_dependent($storage);
}

1;

__END__
